let Genode = env:DHALL_GENODE

in    λ ( _
        : {}
        )
    → { test-pci =
          Genode.Init.Start::{
          , binary = "test-pci"
          , resources = { caps = 96, ram = Genode.units.MiB 2 }
          , routes = [ Genode.ServiceRoute.child "Platform" "platform_drv" ]
          }
      , acpi_report_rom =
          Genode.Init.Start::{
          , binary = "report_rom"
          , resources = { caps = 96, ram = Genode.units.MiB 2 }
          , provides = [ "ROM", "Report" ]
          , config =
              Genode.Prelude.XML.text
                ''
                <config system="yes">
                	<policy label="smbios_decoder -> smbios_table" report="acpi_drv -> smbios_table"/>
                	<policy label="platform_drv -> acpi"           report="acpi_drv -> acpi"/>
                </config>
                ''
          }
      , acpi_drv =
          Genode.Init.Start::{
          , binary = "acpi_drv"
          , resources = { caps = 400, ram = Genode.units.MiB 4 }
          , constrainPhys = True
          , provides = [ "Platform", "Acpi" ]
          , routes =
              [ Genode.ServiceRoute.child "Report" "acpi_report_rom"
              , Genode.ServiceRoute.parent "IRQ"
              , Genode.ServiceRoute.parent "IO_MEM"
              , Genode.ServiceRoute.parent "IO_PORT"
              ]
          }
      , platform_drv =
          Genode.Init.Start::{
          , binary = "platform_drv"
          , resources = { caps = 800, ram = Genode.units.MiB 4 }
          , constrainPhys = True
          , provides = [ "Platform", "Acpi" ]
          , routes =
              [ Genode.ServiceRoute.parent "Timer"
              , Genode.ServiceRoute.parent "IRQ"
              , Genode.ServiceRoute.parent "IO_MEM"
              , Genode.ServiceRoute.parent "IO_PORT"
              , Genode.ServiceRoute.childLabel
                  "ROM"
                  "acpi_report_rom"
                  (Some "acpi")
                  (None Text)
              ]
          , config =
              Genode.Prelude.XML.text
                ''
                <config>
                	<policy label_prefix="test-pci"> <pci class="ALL"/> </policy>
                </config>
                ''
          }
      }
